<?php

/**
 * @file
 * Contains \Drupal\miniorange_saml_idp\Form\MiniorangeSPInformation.
 */

namespace Drupal\miniorange_saml_idp\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Render\Markup;
use Drupal\miniorange_saml_idp\Utilities;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpConstants;

class MiniorangeSPInformation extends FormBase
{
  public function getFormId() {
    return 'miniorange_sp_setup';
  }
  public function buildForm(array $form, FormStateInterface $form_state)
  {
      global $base_url;
      Utilities::visual_tour_start($form, $form_state);
      $login_url = $base_url . '/initiatelogon';
      $issuer = $base_url . '/?q=admin/config/people/miniorange_saml_idp/';
      $module_path = \Drupal::service('extension.list.module')->getPath('miniorange_saml_idp');
      $issuer_id = isset($issuer_id) && !empty($issuer_id)? $issuer_id:$issuer;

      $form['markup_library'] = array(
        '#attached' => array(
          'library' => array(
            'miniorange_saml_idp/miniorange_saml_idp.admin',
          )
        ),
      );

      $form['markup_style_1'] = array(
            '#attached' => array(
                'library' => array(
                    'miniorange_saml_idp/miniorange_saml_idp_copy.icon',
                )
            ),
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">',
      );

      /**
       * Create container to hold @IdentityProviderMetadata form elements.
       */
      $form['mo_saml_identity_provider_metadata'] = array(
          '#type' => 'fieldset',
          //'#title' => t('Identity Provider Metadata'),
          '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
      );

      $form['mo_saml_identity_provider_metadata']['miniorange_saml_idp_div_st'] = array(
          '#markup' => t('<div class="mo_saml_idp_font_for_heading">Identity Provider Metadata</div>
                <a id="Restart_moTour" class="mo_idp_btn mo_idp_btn-danger mo_idp_btn-sm miniorange_float_right" onclick="Restart_moTour()">Take a Tour</a><p style="clear: both"></p><hr><br>'),
      );

      $form['mo_saml_identity_provider_metadata']['mo_saml_metadata_option'] = array(
          '#markup' => t('<div class="mo_saml_font_idp_setup_for_heading">Provide this module information to your Service Provider team.<br> You can choose any one of the below options:</div>
                             <br><b>a) Provide this metadata URL to your Service Provider:</b><br>'),
      );

      $form['mo_saml_identity_provider_metadata']['markup_idp_sp_2'] = array(
          '#markup' => t('<div>
                            <div class="mo_idp_mo_saml_highlight_background_url_note" id="idp_metadata_url">
                                <code><b>
                                    <span>
                                        <a target="_blank" href="' . $base_url . '/moidp_metadata">' . $base_url . '/moidp_metadata</a>
                                    </span></b>
                                </code>
                            </div>
                            <img class ="mo_idp_fa mo_idp_fa-fw mo_idp_fa-lg mo_idp_fa-copy mo_idp_mo_copy mo_idp_mo-margin" src="'. $base_url.'/'.$module_path . '/includes/images/copy-regular.svg">
                        </div>'),
      );

      $form['mo_saml_identity_provider_metadata']['mo_saml_download_btn_title'] = array(
          '#markup' => t('<br><br><div id="download_metadata_file"><b>b) Download the Module XML metadata and upload it on your Service Provider : </b>
                                <span><a href="' . $base_url . '/moidp_metadata_download" class="mo_idp_btn mo_idp_btn-primary mo_idp_btn-sm">Download XML Metadata</a></span>
                        </div><br><br><div><b>c) Provide the following information to your Service Provider. Copy it and keep it handy.</b></div><br>'),
      );

      $idp_Entity = [
          'data' => Markup::create( $issuer_id )
      ];
      $saml_login_url = [
          'data' => Markup::create( $login_url )
      ];
      $saml_logout_url = [
          'data' => Markup::create( 'Available in </b><a href="' . $base_url . MiniorangeSAMLIdpConstants::LICENSE_PAGE_URL .'">Premium</a> version.' )
      ];
      $certificate = [
            'data' => Markup::create('<a href="' . $base_url . '/moidp_certificate_download">Click Here</a> to download X509 certificate.')
      ];
      $response_signed = [
          'data' => Markup::create('Available in </b><a href="' . $base_url . MiniorangeSAMLIdpConstants::LICENSE_PAGE_URL .'">Premium</a> version.' )
      ];
      $assertion_signed = [
          'data' => Markup::create('You can choose to sign your assertion in <a href="' . $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup">Service Provider Setup</a>' )
      ];
      $NameIDFormat = [
          'data' => Markup::create('You can choose nameID format in <a href="' . $base_url . '/admin/config/people/miniorange_saml_idp/idp_setup">Service Provider Setup</a>' )
      ];

      $mo_table_content = array (
          array( 'IDP Entity ID/Issuer', $idp_Entity ),
          array( 'SAML Login URL', $saml_login_url ),
          array( 'SAML Logout URI', $saml_logout_url ),
          array( 'Certificate (Optional)', $certificate ),
          array( 'Response Signed', $response_signed ),
          array( 'Assertion Signed', $assertion_signed ),
          array( 'NameID Format', $NameIDFormat),
      );

      $form['mo_saml_identity_provider_metadata']['mo_dfg_saml_attrs_list_idp'] = array(
          '#type' => 'table',
          '#header'=> array( 'ATTRIBUTE', 'VALUE' ),
          '#rows' => $mo_table_content,
          '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a>'),
          '#responsive' => TRUE ,
          '#sticky'=> TRUE,
          '#size'=> 2,
          '#suffix' => '</div>'
      );

      Utilities::spConfigGuide($form, $form_state);

      return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {

  }
}
